@@ -7,16 +7,19 @@ This program can be distributed under the terms of the GNU GPLv3.
 '''
 
 from .logging import logging, QuietError, setup_logging
-from . import CURRENT_FS_REV, REV_VER_MAP, PICKLE_PROTOCOL
-from .backends.common import NoSuchObject
+from . import CURRENT_FS_REV, REV_VER_MAP, PICKLE_PROTOCOL, BUFSIZE
 from .backends.comprenc import ComprencBackend
 from .database import Connection
 from .common import (get_backend_cachedir, get_seq_no, stream_write_bz2,
-                     stream_read_bz2, is_mounted, get_backend, pretty_print_size)
+                     stream_read_bz2, is_mounted, AsyncFn, get_backend,
+                     get_backend_factory, pretty_print_size, split_by_n,
+                     handle_on_return)
 from .metadata import restore_metadata, cycle_metadata, dump_metadata
 from .parse_args import ArgumentParser
 from datetime import datetime as Datetime
 from getpass import getpass
+from base64 import b64encode
+from queue import Queue, Full as QueueFull
 import os
 import pickle
 import shutil
@@ -24,7 +27,6 @@ import sys
 import tempfile
 import textwrap
 import time
-import atexit
 
 log = logging.getLogger(__name__)
 
@@ -46,14 +48,16 @@ def parse_args(args):
                                        help='may be either of')
     subparsers.add_parser("passphrase", help="change file system passphrase",
                           parents=[pparser])
-    subparsers.add_parser("upgrade", help="upgrade file system to newest revision",
-                          parents=[pparser])
     subparsers.add_parser("clear", help="delete file system and all data",
                           parents=[pparser])
     subparsers.add_parser("download-metadata",
                           help="Interactively download metadata backups. "
                                "Use only if you know what you are doing.",
                           parents=[pparser])
+    sparser = subparsers.add_parser("upgrade", help="upgrade file system to newest revision",
+                          parents=[pparser])
+    sparser.add_argument("--threads", type=int, default=20,
+                        help='Number of threads to use')
 
     parser.add_debug()
     parser.add_quiet()
@@ -83,24 +87,18 @@ def main(args=None):
         raise QuietError('Can not work on mounted file system.')
 
     if options.action == 'clear':
-        backend = get_backend(options, plain=True)
-        atexit.register(backend.close)
-        return clear(backend,
-                     get_backend_cachedir(options.storage_url, options.cachedir))
-
-    backend = get_backend(options)
-    atexit.register(backend.close)
+        with get_backend(options, plain=True) as backend:
+            return clear(backend, options)
 
     if options.action == 'upgrade':
-        return upgrade(backend, get_backend_cachedir(options.storage_url,
-                                                   options.cachedir))
-
-    if options.action == 'passphrase':
-        return change_passphrase(backend)
-
-    if options.action == 'download-metadata':
-        return download_metadata(backend, options.storage_url)
+        return upgrade(options)
+    
+    with get_backend(options) as backend:
+        if options.action == 'passphrase':
+            return change_passphrase(backend)
 
+        if options.action == 'download-metadata':
+            return download_metadata(backend, options.storage_url)
 
 def download_metadata(backend, storage_url):
     '''Download old metadata backups'''
@@ -184,8 +182,7 @@ def change_passphrase(backend):
     backend['s3ql_passphrase_bak3'] = data_pw
     backend.passphrase = data_pw
 
-
-def clear(backend, cachepath):
+def clear(backend, options):
     print('I am about to delete all data in %s.' % backend,
           'This includes any S3QL file systems as well as any other stored objects.',
           'Please enter "yes" to continue.', '> ', sep='\n', end='')
@@ -195,7 +192,7 @@ def clear(backend, cachepath):
         raise QuietError()
 
     log.info('Deleting...')
-
+    cachepath = get_backend_cachedir(options.storage_url, options.cachedir)
     for suffix in ('.db', '.params'):
         name = cachepath + suffix
         if os.path.exists(name):
@@ -224,51 +221,20 @@ def get_old_rev_msg(rev, prog):
         ''' % { 'version': REV_VER_MAP[rev],
                 'prog': prog })
 
-def upgrade_monkeypatch(backend):
-    '''Monkeypatch ComprencBackend instance *backend* for upgrade
 
-    Monkeypatch *backend*, so that we can read the current
-    s3ql_metadata object without getting a CorruptedObjectError. This would
-    happen because previous S3QL versions don't update the object
-    key stored in the object on rename, and the metadata object is
-    created as s3ql_metadata_new``.
-    '''
-    from base64 import b64decode
-
-    verify_meta_orig = backend._verify_meta
-    def _verify_meta_new(key, metadata):
-        if (metadata['encryption'] != 'None'
-            and key == 's3ql_metadata'):
-            stored_key = b64decode(metadata['object_id']).decode('utf-8')
-            if stored_key == 's3ql_metadata_new':
-                key = stored_key
-        return verify_meta_orig(key, metadata)
-    backend._verify_meta = _verify_meta_new
-
-def upgrade(backend, cachepath):
+@handle_on_return
+def upgrade(options, on_return):
     '''Upgrade file system to newest revision'''
 
     log.info('Getting file system parameters..')
+    cachepath = get_backend_cachedir(options.storage_url, options.cachedir)
 
-    upgrade_monkeypatch(backend)
-
-    # Get sequence number and check for *really* old S3QL version
-    seq_nos = list(backend.list('s3ql_seq_no_'))
-    if (seq_nos[0].endswith('.meta')
-        or seq_nos[0].endswith('.dat')):
-        print(textwrap.dedent('''
-            File system revision too old to upgrade!
-
-            You need to use an older S3QL version to upgrade to a more recent
-            revision before you can use this version to upgrade to the newest
-            revision.
-            '''))
-        print(get_old_rev_msg(11 + 1, 's3qladm'))
-        raise QuietError()
-    seq_no = get_seq_no(backend)
+    backend_factory = get_backend_factory(options)
+    backend = on_return.enter_context(backend_factory())
 
     # Check for cached metadata
     db = None
+    seq_no = get_seq_no(backend)
     if os.path.exists(cachepath + '.params'):
         with open(cachepath + '.params', 'rb') as fh:
             param = pickle.load(fh)
@@ -358,20 +324,19 @@ def upgrade(backend, cachepath):
     param['last-modified'] = time.time()
     param['seq_no'] += 1
 
-    # In this update, we have added a new metadata format. Old objects can still
-    # be read, so we don't need to do any conversion work. However, accessing
-    # (including renaming) encrypted metadata backups will fail because the
-    # embedded object key does not agree with the key under which the object is
-    # stored. Therefore, we need to make sure that we don't try to touch them at
-    # all.
-    if isinstance(backend, ComprencBackend) and backend.passphrase:
-        plain_backend = backend.backend
-        for i in range(10)[::-1]:
-            try:
-                plain_backend.rename("s3ql_metadata_bak_%d" % i,
-                                     "s3ql_metadata_bak_%d_pre21" % i)
-            except NoSuchObject:
-                continue
+    # Ensure that there are backups of the master key
+    if backend.passphrase is not None:
+        data_pw = backend.passphrase
+        backend.passphrase = backend.fs_passphrase
+        for i in range(1,4):
+            obj_id = 's3ql_passphrase_bak%d' % i
+            if obj_id not in backend:
+                backend[obj_id] = data_pw
+        backend.passphrase = data_pw
+
+    # Upgrade all objects, so that we can remove legacy conversion
+    # routines in the next release.
+    update_obj_metadata(backend, backend_factory, db, options.threads)
 
     log.info('Dumping metadata...')
     with tempfile.TemporaryFile() as fh:
@@ -399,39 +364,156 @@ def upgrade(backend, cachepath):
     db.execute('ANALYZE')
     db.execute('VACUUM')
 
-    print(textwrap.dedent('''\
-        File system upgrade complete.
+    print('File system upgrade complete.')
 
-        It is strongly recommended to run the s3ql_verify command with the
-        --data option as soon as possible. This is necessary to ensure that the
-        upgrade to the next (2.11) S3QL release will run smoothly.'''))
+    if backend.passphrase is not None:
+        print('Please store the following master key in a safe location. It allows ',
+              'decryption of the S3QL file system in case the storage objects holding ',
+              'this information get corrupted:',
+              '---BEGIN MASTER KEY---',
+              ' '.join(split_by_n(b64encode(backend.passphrase).decode(), 4)),
+              '---END MASTER KEY---',
+              sep='\n')
 
-# This should be used on the *next* fs revision update to ensure that all
-# objects conform to newest standards, so we can drop the legacy routines from
-# backend/common.py as well as the legacy metadata handling in backends/s3c.py
-def update_obj_metadata(backend, db):
+@handle_on_return
+def update_obj_metadata(backend, backend_factory, db,
+                        thread_count, on_return):
     '''Upgrade metadata of storage objects'''
 
-    # TODO: Don't forget the s3ql_passphrase object and (maybe one) backup!
-
-    if not isinstance(backend, ComprencBackend):
-        # Need to make sure that all plain metadata is using
-        # pickle format
-        raise RuntimeError('not implemented yet')
-
     plain_backend = backend.backend
-    for (obj_id,) in db.query('SELECT id FROM obj_ids'):
-        meta = plain_backend.lookup('s3ql_data_%d' % obj_id)
-        if 'format_version' in meta:
+
+    # No need to update sequence number, since we are going to
+    # write out a new one after the upgrade.
+    if backend.passphrase is None:
+        extra_objects = { 's3ql_metadata' }
+    else:
+        extra_objects = { 's3ql_metadata',
+                          's3ql_passphrase', 's3ql_passphrase_bak1',
+                          's3ql_passphrase_bak2', 's3ql_passphrase_bak3' }
+
+    for i in range(30):
+        obj_id = 's3ql_metadata_bak_%d' % i
+        if obj_id in plain_backend:
+            extra_objects.add(obj_id)
+
+    def yield_objects():
+        for (id_,) in db.query('SELECT id FROM objects'):
+            yield 's3ql_data_%d' % id_
+        for obj_id in extra_objects:
+            yield obj_id
+    total = db.get_val('SELECT COUNT(id) FROM objects') + len(extra_objects)
+
+    queue = Queue(maxsize=thread_count)
+    threads = []
+    for _ in range(thread_count):
+        t = AsyncFn(upgrade_loop, queue, on_return.push(backend_factory()))
+        # Don't wait for worker threads, gives deadlock if main thread
+        # terminates with exception
+        t.daemon = True
+        t.start()
+        threads.append(t)
+
+    # Updating this value is prone to race conditions. However,
+    # we don't care because this is for an approximate progress
+    # output only.
+    queue.rewrote_size = 0
+    stamp = 0
+    for (i, obj_id) in enumerate(yield_objects()):
+        stamp2 = time.time()
+        if stamp2 - stamp > 1:
+            sys.stdout.write('\r..processed %d/%d objects (%.1f%%, %s rewritten)..'
+                             % (i, total, i/total*100,
+                                pretty_print_size(queue.rewrote_size)))
+            sys.stdout.flush()
+            stamp = stamp2
+
+            # Terminate early if any thread failed with an exception
+            for t in threads:
+                if not t.is_alive():
+                    t.join_and_raise()
+
+        # Avoid blocking if all threads terminated
+        while True:
+            try:
+                queue.put(obj_id, timeout=1)
+            except QueueFull:
+                pass
+            else:
+                break
+            for t in threads:
+                if not t.is_alive():
+                    t.join_and_raise()
+
+    queue.maxsize += len(threads)
+    for t in threads:
+        queue.put(None)
+
+    for t in threads:
+        t.join_and_raise()
+
+    sys.stdout.write('\n')
+
+def upgrade_loop(queue, backend):
+    plain_backend = backend.backend
+
+    while True:
+        obj_id = queue.get()
+        if obj_id is None:
+            break
+
+        meta = plain_backend.lookup(obj_id)
+        if meta.get('format_version', 0) == 2:
             continue
+
+        # For important objects, we make a copy first (just to be safe)
+        if not obj_id.startswith('s3ql_data'):
+            plain_backend.copy(obj_id, 's3ql_pre2.13' + obj_id[4:])
+
+        # When reading passphrase objects, we have to use the
+        # "outer" password
+        if obj_id.startswith('s3ql_passphrase'):
+            data_pw = backend.passphrase
+            backend.passphrase = backend.fs_passphrase
+
         meta = backend._convert_legacy_metadata(meta)
-
         if meta['encryption'] == 'AES':
-            #rewrite_legacy_object(obj_id, meta)
-            raise RuntimeError("Don't know how to convert legacy objects yet")
+            # Two statements to reduce likelihood of update races
+            size = rewrite_legacy_object(backend, obj_id)
+            queue.rewrote_size += size
         else:
-            plain_backend.update_meta('s3ql_data_%d' % obj_id, meta)
+            plain_backend.update_meta(obj_id, meta)
 
+        if obj_id.startswith('s3ql_passphrase'):
+            backend.passphrase = data_pw
+
+def rewrite_legacy_object(backend, obj_id):
+    with tempfile.TemporaryFile() as tmpfh:
+
+        # Read object
+        def do_read(fh):
+            tmpfh.seek(0)
+            tmpfh.truncate()
+            while True:
+                buf = fh.read(BUFSIZE)
+                if not buf:
+                    break
+                tmpfh.write(buf)
+            return fh.metadata
+
+        meta = backend.perform_read(do_read, obj_id)
+
+        # Write object
+        def do_write(fh):
+            tmpfh.seek(0)
+            while True:
+                buf = tmpfh.read(BUFSIZE)
+                if not buf:
+                    break
+                fh.write(buf)
+            return fh
+        out_fh = backend.perform_write(do_write, obj_id, meta)
+
+        return out_fh.get_obj_size()
 
 if __name__ == '__main__':
     main(sys.argv[1:])
