@@ -32,7 +32,8 @@ class Backend(s3c.Backend):
     may or may not be available and can be queried for with instance methods.
     """
 
-    known_options = s3c.Backend.known_options | { 'sse', 'rrs' }
+    known_options = ((s3c.Backend.known_options | { 'sse', 'rrs' })
+                     - {'dumb-copy', 'disable-expect100'})
 
     def __init__(self, storage_url, login, password, options):
         super().__init__(storage_url, login, password, options)
@@ -73,7 +74,7 @@ class Backend(s3c.Backend):
 
     @copy_ancestor_docstring
     def delete_multi(self, keys, force=False):
-        log.debug('delete_multi(%s)', keys)
+        log.debug('started with %s', keys)
 
         while len(keys) > 0:
             tmp = keys[:MAX_KEYS]
