@@ -7,10 +7,11 @@ This program can be distributed under the terms of the GNU GPLv3.
 '''
 
 from ..logging import logging, QuietError # Ensure use of custom logger class
-from .. import PICKLE_PROTOCOL, BUFSIZE
+from .. import BUFSIZE
 from .common import (AbstractBackend, NoSuchObject, retry, AuthorizationError,
                      AuthenticationError, DanglingStorageURLError, retry_generator,
-                     get_proxy, get_ssl_context, CorruptedObjectError, safe_unpickle)
+                     get_proxy, get_ssl_context, CorruptedObjectError, safe_unpickle,
+                     checksum_basic_mapping)
 from ..inherit_docstrings import (copy_ancestor_docstring, prepend_ancestor_docstring,
                                   ABCDocstMeta)
 from io import BytesIO
@@ -19,10 +20,13 @@ from dugong import (HTTPConnection, is_temp_network_error, BodyFollowing, CaseIn
                     UnsupportedResponse, ConnectionClosed)
 from base64 import b64encode, b64decode
 from email.utils import parsedate_tz, mktime_tz
-from urllib.parse import urlsplit
+from ast import literal_eval
+from urllib.parse import urlsplit, quote, unquote
 import defusedxml.cElementTree as ElementTree
+from itertools import count
 import hashlib
 import os
+import binascii
 import hmac
 import re
 import tempfile
@@ -40,18 +44,13 @@ log = logging.getLogger(__name__)
 class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     """A backend to stored data in some S3 compatible storage service.
 
-    This class uses standard HTTP connections to connect to GS.
-
     The backend guarantees only immediate get after create consistency.
-
-    If the *expect_100c* class variable is True, the 'Expect: 100-continue'
-    header is used to check for error codes before uploading payload data.
     """
 
-    use_expect_100c = True
     xml_ns_prefix = '{http://s3.amazonaws.com/doc/2006-03-01/}'
     hdr_prefix = 'x-amz-'
-    known_options = {'no-ssl', 'ssl-ca-path', 'tcp-timeout'}
+    known_options = {'no-ssl', 'ssl-ca-path', 'tcp-timeout',
+                     'dumb-copy', 'disable-expect100'}
 
     def __init__(self, storage_url, login, password, options):
         '''Initialize backend object
@@ -84,6 +83,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     def has_native_rename(self):
         return False
 
+    # NOTE: ! This function is also used by the swift backend !
     @copy_ancestor_docstring
     def reset(self):
         if self.conn.response_pending() or self.conn._out_remaining:
@@ -123,7 +123,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
         conn =  HTTPConnection(self.hostname, self.port, proxy=self.proxy,
                                ssl_context=self.ssl_context)
-        conn.timeout = self.options.get('tcp-timeout', 10)
+        conn.timeout = int(self.options.get('tcp-timeout', 10))
         return conn
 
     # This method is also used implicitly for the retry handling of
@@ -152,6 +152,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
         return False
 
+    # NOTE: ! This function is also used by the swift backend. !
     def _dump_response(self, resp, body=None):
         '''Return string representation of server response
 
@@ -175,6 +176,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
                                     '\n'.join('%s: %s' % x for x in resp.headers.items()),
                                     body.decode('utf-8', errors='backslashreplace'))
 
+    # NOTE: ! This function is also used by the swift backend. !
     def _assert_empty_response(self, resp):
         '''Assert that current response body is empty'''
 
@@ -193,7 +195,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     @retry
     @copy_ancestor_docstring
     def delete(self, key, force=False):
-        log.debug('delete(%s)', key)
+        log.debug('started with %s', key)
         try:
             resp = self._do_request('DELETE', '/%s%s' % (self.prefix, key))
             self._assert_empty_response(resp)
@@ -206,15 +208,15 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     @retry_generator
     @copy_ancestor_docstring
     def list(self, prefix='', start_after=''):
-        log.debug('list(%s, %s): start', prefix, start_after)
+        log.debug('started with %s, %s', prefix, start_after)
 
         keys_remaining = True
-        marker = start_after
+        marker = self.prefix + start_after
         prefix = self.prefix + prefix
         ns_p = self.xml_ns_prefix
 
         while keys_remaining:
-            log.debug('list(%s): requesting with marker=%s', prefix, marker)
+            log.debug('requesting with marker=%s', marker)
 
             keys_remaining = None
             resp = self._do_request('GET', '/', query_string={ 'prefix': prefix,
@@ -225,10 +227,10 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
                 raise RuntimeError('unexpected content type: %s' %
                                    resp.headers['Content-Type'])
 
-            itree = iter(ElementTree.iterparse(self.conn, events=("start", "end")))
-            (event, root) = next(itree)
-
             try:
+                itree = iter(ElementTree.iterparse(self.conn, events=("start", "end")))
+                (event, root) = next(itree)
+
                 for (event, el) in itree:
                     if event != 'end':
                         continue
@@ -241,6 +243,12 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
                         yield marker[len(self.prefix):]
                         root.clear()
 
+            except Exception as exc:
+                if is_temp_network_error(exc):
+                    # We probably can't use the connection anymore
+                    self.conn.disconnect()
+                raise
+
             except GeneratorExit:
                 # Need to read rest of response
                 self.conn.discard()
@@ -252,7 +260,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     @retry
     @copy_ancestor_docstring
     def lookup(self, key):
-        log.debug('lookup(%s)', key)
+        log.debug('started with %s', key)
 
         try:
             resp = self._do_request('HEAD', '/%s%s' % (self.prefix, key))
@@ -268,7 +276,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     @retry
     @copy_ancestor_docstring
     def get_size(self, key):
-        log.debug('get_size(%s)', key)
+        log.debug('started with %s', key)
 
         try:
             resp = self._do_request('HEAD', '/%s%s' % (self.prefix, key))
@@ -295,7 +303,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
         try:
             meta = self._extractmeta(resp, key)
-        except BadDigestError:
+        except (BadDigestError, CorruptedObjectError):
             # If there's less than 64 kb of data, read and throw
             # away. Otherwise re-establish connection.
             if resp.length is not None and resp.length < 64*1024:
@@ -313,44 +321,61 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
         when its `close` method is called.
         """
 
-        log.debug('open_write(%s): start', key)
-
-        if metadata is None:
-            metadata = dict()
-        elif not isinstance(metadata, dict):
-            raise TypeError('*metadata*: expected dict or None, got %s' % type(metadata))
+        log.debug('started with %s', key)
 
         headers = CaseInsensitiveDict()
         if extra_headers is not None:
             headers.update(extra_headers)
+        if metadata is None:
+            metadata = dict()
         self._add_meta_headers(headers, metadata)
 
         return ObjectW(key, self, headers)
 
-    # Note! This function is also used by the swift backend.
+    # NOTE: ! This function is also used by the swift backend. !
     def _add_meta_headers(self, headers, metadata, chunksize=255):
 
-        # We don't store the metadata keys directly, because HTTP headers
-        # are case insensitive (so the server may change capitalization)
-        # and we may run into length restrictions.
-        meta_buf = b64encode(pickle.dumps(metadata, PICKLE_PROTOCOL))
+        hdr_count = 0
+        length = 0
+        for key in metadata.keys():
+            if not isinstance(key, str):
+                raise ValueError('dict keys must be str, not %s' % type(key))
+            val = metadata[key]
+            if (not isinstance(val, (str, bytes, int, float, complex, bool))
+                and val is not None):
+                raise ValueError('value for key %s (%s) is not elementary' % (key, val))
 
-        headers[self.hdr_prefix + 'meta-format'] = 'pickle'
-        headers[self.hdr_prefix + 'meta-md5'] = md5sum_b64(meta_buf)
+            if isinstance(val, (bytes, bytearray)):
+                val = b64encode(val)
 
-        i = 0
-        while i*chunksize < len(meta_buf):
-            p = meta_buf[i*chunksize:(i+1)*chunksize].decode('us-ascii')
-            headers[self.hdr_prefix + 'meta-data-%02d' % i] = p
-            i += 1
+            buf = ('%s: %s,' % (repr(key), repr(val)))
+            buf = quote(buf, safe='!@#$^*()=+/?-_\'"><\\| `.,;:~')
+            if len(buf) < chunksize:
+                headers['%smeta-%03d' % (self.hdr_prefix, hdr_count)] = buf
+                hdr_count += 1
+                length += 4 + len(buf)
+            else:
+                i = 0
+                while i*chunksize < len(buf):
+                    k = '%smeta-%03d' % (self.hdr_prefix, hdr_count)
+                    v = buf[i*chunksize:(i+1)*chunksize]
+                    headers[k] = v
+                    i += 1
+                    hdr_count += 1
+                    length += 4 + len(buf)
+
+            if length > 2048:
+                raise ValueError('Metadata too large')
+
+        assert hdr_count <= 999
+        md5 = b64encode(checksum_basic_mapping(metadata)).decode('ascii')
+        headers[self.hdr_prefix + 'meta-format'] = 'raw2'
+        headers[self.hdr_prefix + 'meta-md5'] = md5
 
     @retry
     @copy_ancestor_docstring
     def copy(self, src, dest, metadata=None, extra_headers=None):
-        log.debug('copy(%s, %s): start', src, dest)
-
-        if not (metadata is None or isinstance(metadata, dict)):
-            raise TypeError('*metadata*: expected dict or None, got %s' % type(metadata))
+        log.debug('started with %s, %s', src, dest)
 
         headers = CaseInsensitiveDict()
         if extra_headers is not None:
@@ -372,6 +397,9 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
         # When copying, S3 may return error despite a 200 OK status
         # http://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectCOPY.html
         # https://doc.s3.amazonaws.com/proposals/copy.html
+        if self.options.get('dumb-copy', False):
+            self.conn.discard()
+            return
         body = self.conn.readall()
         root = self._parse_xml_response(resp, body)
         if root.tag == self.xml_ns_prefix + 'CopyObjectResult':
@@ -386,15 +414,13 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
     @copy_ancestor_docstring
     def update_meta(self, key, metadata):
-        if not isinstance(metadata, dict):
-            raise TypeError('*metadata*: expected dict, got %s' % type(metadata))
         self.copy(key, key, metadata)
 
     def _do_request(self, method, path, subres=None, query_string=None,
                     headers=None, body=None):
         '''Send request, read and return response object'''
 
-        log.debug('preparing %s %s?%s, qs=%s', method, path, subres, query_string)
+        log.debug('started with %s %s?%s, qs=%s', method, path, subres, query_string)
 
         if headers is None:
             headers = CaseInsensitiveDict()
@@ -540,6 +566,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
         return tree
 
+    # NOTE: ! This function is also used by the swift backend. !
     @prepend_ancestor_docstring
     def clear(self):
         """
@@ -553,7 +580,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             if no != 0 and no % 1000 == 0:
                 log.info('clear(): deleted %d objects so far..', no)
 
-            log.debug('clear(): deleting key %s', s3key)
+            log.debug('started with %s', s3key)
 
             # Ignore missing objects when clearing bucket
             self.delete(s3key, True)
@@ -632,16 +659,17 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             assert resp.path == path
             return resp
 
+        use_expect_100c = not self.options.get('disable-expect100', False)
         try:
-            log.debug('_send_request(): %s %s', method, path)
+            log.debug('sending %s %s', method, path)
             if body is None or isinstance(body, (bytes, bytearray, memoryview)):
                 self.conn.send_request(method, path, body=body, headers=headers)
             else:
                 body_len = os.fstat(body.fileno()).st_size
-                self.conn.send_request(method, path, expect100=self.use_expect_100c,
+                self.conn.send_request(method, path, expect100=use_expect_100c,
                                        headers=headers, body=BodyFollowing(body_len))
 
-                if self.use_expect_100c:
+                if use_expect_100c:
                     resp = read_response()
                     if resp.status != 100: # Error
                         return resp
@@ -676,44 +704,73 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     def close(self):
         self.conn.disconnect()
 
-    # Note! This function is also used by the swift backend.
-    def _extractmeta(self, resp, key):
+    # NOTE: ! This function is also used by the swift backend !
+    def _extractmeta(self, resp, obj_key):
         '''Extract metadata from HTTP response object'''
 
-        meta = dict()
-        pattern = re.compile(r'^%smeta-(.+)$' % re.escape(self.hdr_prefix),
-                             re.IGNORECASE)
-        format_ = 'raw'
-        for (name, val) in resp.headers.items():
-            # HTTP headers are case-insensitive and pre 2.x S3QL versions metadata
-            # names verbatim (and thus loose capitalization info), so we force lower
-            # case (since we know that this is the original capitalization).
-            name = name.lower()
+        format_ = resp.headers.get('%smeta-format' % self.hdr_prefix, 'raw')
+        if format_ in ('raw', 'pickle'):
+            meta = CaseInsensitiveDict()
+            pattern = re.compile(r'^%smeta-(.+)$' % re.escape(self.hdr_prefix),
+                                 re.IGNORECASE)
+            for fname in resp.headers:
+                hit = pattern.search(fname)
+                if hit:
+                    meta[hit.group(1)] = resp.headers[fname]
 
-            hit = pattern.search(name)
-            if not hit:
-                continue
+            if format_ == 'raw':
+                return meta
 
-            if hit.group(1) == 'format':
-                format_ = val
-            else:
-                meta[hit.group(1)] = val
-
-        if format_ == 'pickle':
-            buf = ''.join(meta[x] for x in sorted(meta)
-                          if x.startswith('data-'))
+            # format_ == pickle
+            buf = ''.join(meta[x] for x in sorted(meta) if x.lower().startswith('data-'))
             if 'md5' in meta and md5sum_b64(buf.encode('us-ascii')) != meta['md5']:
-                log.warning('MD5 mismatch in metadata for %s', key)
-                raise BadDigestError('BadDigest',
-                                     'Meta MD5 for %s does not match' % key)
+                log.warning('MD5 mismatch in metadata for %s', obj_key)
+                raise BadDigestError('BadDigest', 'Meta MD5 for %s does not match' % obj_key)
             try:
                 return safe_unpickle(b64decode(buf), encoding='latin1')
+            except binascii.Error:
+                raise CorruptedObjectError('Corrupted metadata, b64decode failed')
             except pickle.UnpicklingError as exc:
                 raise CorruptedObjectError('Corrupted metadata, pickle says: %s' % exc)
-        elif format_ == 'raw': # No MD5 available
-            return meta
-        else:
-            raise RuntimeError('Unknown metadata format: %s' % format_)
+
+        elif format_ != 'raw2': # Current
+            raise RuntimeError('Unknown metadata format %s for key %s'
+                               % (format_, obj_key))
+
+        parts = []
+        for i in count():
+            # Headers is an email.message object, so indexing it
+            # would also give None instead of KeyError
+            part = resp.headers.get('%smeta-%03d' % (self.hdr_prefix, i), None)
+            if part is None:
+                break
+            parts.append(part)
+        buf = unquote(''.join(parts))
+        meta = literal_eval('{ %s }' % buf)
+
+        # Decode bytes values
+        for (k,v) in meta.items():
+            if not isinstance(v, bytes):
+                continue
+            try:
+                meta[k] = b64decode(v)
+            except binascii.Error:
+                # This should trigger a MD5 mismatch below
+                meta[k] = None
+
+        # Check MD5. There is a case to be made for treating a mismatch as a
+        # `CorruptedObjectError` rather than a `BadDigestError`, because the MD5
+        # sum is not calculated on-the-fly by the server but stored with the
+        # object, and therefore does not actually verify what the server has
+        # sent over the wire. However, it seems more likely for the data to get
+        # accidentally corrupted in transit than to get accidentally corrupted
+        # on the server (which hopefully checksums its storage devices).
+        md5 = b64encode(checksum_basic_mapping(meta)).decode('ascii')
+        if md5 != resp.headers.get('%smeta-md5' % self.hdr_prefix, None):
+            log.warning('MD5 mismatch in metadata for %s', obj_key)
+            raise BadDigestError('BadDigest', 'Meta MD5 for %s does not match' % obj_key)
+
+        return meta
 
 class ObjectR(object):
     '''An S3 object open for reading'''
@@ -745,6 +802,10 @@ class ObjectR(object):
         if size == 0:
             return b''
 
+        # This may raise an exception, in which case we probably can't
+        # re-use the connection. However, we rely on the caller
+        # to still close the file-like object, so that we can do
+        # cleanup in close().
         buf = self.backend.conn.read(size)
         self.md5.update(buf)
 
@@ -832,7 +893,7 @@ class ObjectW(object):
         # Access to protected member ok
         #pylint: disable=W0212
 
-        log.debug('ObjectW(%s).close(): start', self.key)
+        log.debug('started with %s', self.key)
 
         if self.closed:
             # still call fh.close, may have generated an error before
@@ -889,13 +950,11 @@ def get_S3Error(code, msg, headers=None):
 
     return class_(code, msg, headers)
 
-
 def md5sum_b64(buf):
     '''Return base64 encoded MD5 sum'''
 
     return b64encode(hashlib.md5(buf).digest()).decode('ascii')
 
-
 def _parse_retry_after(header):
     '''Parse headers for Retry-After value'''
 
