@@ -34,17 +34,17 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
     The backend guarantees get after create consistency, i.e. a newly created
     object will be immediately retrievable.
-
-    If the *expect_100c* attribute is True, the 'Expect: 100-continue' header is
-    used to check for error codes before uploading payload data.
     """
 
-    use_expect_100c = True
     hdr_prefix = 'X-Object-'
-    known_options = {'no-ssl', 'ssl-ca-path', 'tcp-timeout'}
+    known_options = {'no-ssl', 'ssl-ca-path', 'tcp-timeout', 'disable-expect100'}
 
     _add_meta_headers = s3c.Backend._add_meta_headers
     _extractmeta = s3c.Backend._extractmeta
+    _assert_empty_response = s3c.Backend._assert_empty_response
+    _dump_response = s3c.Backend._dump_response
+    clear = s3c.Backend.clear
+    reset = s3c.Backend.reset
 
     def __init__(self, storage_url, login, password, options):
         super().__init__()
@@ -59,10 +59,9 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
         self.password = password
         self.login = login
 
-        if 'no-ssl' in options:
-            self.ssl_context = None
-        else:
-            self.ssl_context = get_ssl_context(options.get('ssl-ca-path', None))
+        # We may need the context even if no-ssl has been specified,
+        # because no-ssl applies only to the authentication URL.
+        self.ssl_context = get_ssl_context(options.get('ssl-ca-path', None))
 
         self._parse_storage_url(storage_url, self.ssl_context)
         self.proxy = get_proxy(self.ssl_context is not None)
@@ -136,51 +135,59 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
         return False
 
-    @retry
     def _get_conn(self):
         '''Obtain connection to server and authentication token'''
 
-        log.debug('_get_conn(): start')
+        log.debug('started')
 
-        conn = HTTPConnection(self.hostname, self.port, proxy=self.proxy,
-                              ssl_context=self.ssl_context)
-        conn.timeout = self.options.get('tcp-timeout', 10)
+        if 'no-ssl' in self.options:
+            ssl_context = None
+        else:
+            ssl_context = self.ssl_context
 
         headers = CaseInsensitiveDict()
         headers['X-Auth-User'] = self.login
         headers['X-Auth-Key'] = self.password
 
-        for auth_path in ('/v1.0', '/auth/v1.0'):
-            log.debug('_get_conn(): GET %s', auth_path)
-            conn.send_request('GET', auth_path, headers=headers)
-            resp = conn.read_response()
+        with HTTPConnection(self.hostname, self.port, proxy=self.proxy,
+                              ssl_context=ssl_context) as conn:
+            conn.timeout = int(self.options.get('tcp-timeout', 10))
 
-            if resp.status in (404, 412):
-                log.debug('_refresh_auth(): auth to %s failed, trying next path', auth_path)
-                conn.discard()
-                continue
+            for auth_path in ('/v1.0', '/auth/v1.0'):
+                log.debug('GET %s', auth_path)
+                conn.send_request('GET', auth_path, headers=headers)
+                resp = conn.read_response()
 
-            elif resp.status == 401:
-                conn.disconnect()
-                raise AuthorizationError(resp.reason)
+                if resp.status in (404, 412):
+                    log.debug('auth to %s failed, trying next path', auth_path)
+                    conn.discard()
+                    continue
 
-            elif resp.status > 299 or resp.status < 200:
-                conn.disconnect()
-                raise HTTPError(resp.status, resp.reason, resp.headers)
+                elif resp.status == 401:
+                    raise AuthorizationError(resp.reason)
 
-            # Pylint can't infer SplitResult Types
-            #pylint: disable=E1103
-            self.auth_token = resp.headers['X-Auth-Token']
-            o = urlsplit(resp.headers['X-Storage-Url'])
-            self.auth_prefix = urllib.parse.unquote(o.path)
-            conn.disconnect()
+                elif resp.status > 299 or resp.status < 200:
+                    raise HTTPError(resp.status, resp.reason, resp.headers)
 
-            conn =  HTTPConnection(o.hostname, o.port, proxy=self.proxy,
-                                  ssl_context=self.ssl_context)
-            conn.timeout = self.options.get('tcp-timeout', 10)
-            return conn
+                # Pylint can't infer SplitResult Types
+                #pylint: disable=E1103
+                self.auth_token = resp.headers['X-Auth-Token']
+                o = urlsplit(resp.headers['X-Storage-Url'])
+                self.auth_prefix = urllib.parse.unquote(o.path)
+                if o.scheme == 'https':
+                    ssl_context = self.ssl_context
+                elif o.scheme == 'http':
+                    ssl_context = None
+                else:
+                    # fall through to scheme used for authentication
+                    pass
 
-        raise RuntimeError('No valid authentication path found')
+                conn =  HTTPConnection(o.hostname, o.port, proxy=self.proxy,
+                                       ssl_context=ssl_context)
+                conn.timeout = int(self.options.get('tcp-timeout', 10))
+                return conn
+
+            raise RuntimeError('No valid authentication path found')
 
     def _do_request(self, method, path, subres=None, query_string=None,
                     headers=None, body=None):
@@ -189,7 +196,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
         This method modifies the *headers* dictionary.
         '''
 
-        log.debug('_do_request(): start with parameters (%r, %r, %r, %r, %r, %r)',
+        log.debug('started with %r, %r, %r, %r, %r, %r',
                   method, path, subres, query_string, headers, body)
 
         if headers is None:
@@ -199,7 +206,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             headers['Content-MD5'] = md5sum_b64(body)
 
         if self.conn is None:
-            log.debug('_do_request(): no active connection, calling _get_conn()')
+            log.debug('no active connection, calling _get_conn()')
             self.conn =  self._get_conn()
 
         # Construct full path
@@ -248,16 +255,18 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     def _do_request_inner(self, method, path, body, headers):
         '''The guts of the _do_request method'''
 
-        log.debug('_do_request_inner(): %s %s', method, path)
+        log.debug('started with %s %s', method, path)
+        use_expect_100c = not self.options.get('disable-expect100', False)
 
         if body is None or isinstance(body, (bytes, bytearray, memoryview)):
             self.conn.send_request(method, path, body=body, headers=headers)
-        else:
-            body_len = os.fstat(body.fileno()).st_size
-            self.conn.send_request(method, path, expect100=self.use_expect_100c,
-                                   headers=headers, body=BodyFollowing(body_len))
+            return self.conn.read_response()
 
-        if self.use_expect_100c:
+        body_len = os.fstat(body.fileno()).st_size
+        self.conn.send_request(method, path, expect100=use_expect_100c,
+                               headers=headers, body=BodyFollowing(body_len))
+
+        if use_expect_100c:
             log.debug('waiting for 100-continue')
             resp = self.conn.read_response()
             if resp.status != 100:
@@ -286,44 +295,10 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
         return self.conn.read_response()
 
-    def _assert_empty_response(self, resp):
-        '''Assert that current response body is empty'''
-
-        buf = self.conn.read(2048)
-        if not buf:
-            return # expected
-
-        # Log the problem
-        self.conn.discard()
-        log.error('Unexpected server response. Expected nothing, got:\n'
-                  '%d %s\n%s\n\n%s', resp.status, resp.reason,
-                  '\n'.join('%s: %s' % x for x in resp.headers.items()),
-                  buf)
-        raise RuntimeError('Unexpected server response')
-
-
-    def _dump_response(self, resp, body=None):
-        '''Return string representation of server response
-
-        Only the beginning of the response body is read, so this is
-        mostly useful for debugging.
-        '''
-
-        if body is None:
-            body = self.conn.read(2048)
-            if body:
-                self.conn.discard()
-        else:
-            body = body[:2048]
-
-        return '%d %s\n%s\n\n%s' % (resp.status, resp.reason,
-                                    '\n'.join('%s: %s' % x for x in resp.headers.items()),
-                                    body.decode('utf-8', errors='backslashreplace'))
-
     @retry
     @copy_ancestor_docstring
     def lookup(self, key):
-        log.debug('lookup(%s)', key)
+        log.debug('started with %s', key)
         if key.endswith(TEMP_SUFFIX):
             raise ValueError('Keys must not end with %s' % TEMP_SUFFIX)
 
@@ -343,7 +318,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     def get_size(self, key):
         if key.endswith(TEMP_SUFFIX):
             raise ValueError('Keys must not end with %s' % TEMP_SUFFIX)
-        log.debug('get_size(%s)', key)
+        log.debug('started with %s', key)
 
         try:
             resp = self._do_request('HEAD', '/%s%s' % (self.prefix, key))
@@ -390,41 +365,24 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
         The returned object will buffer all data and only start the upload
         when its `close` method is called.
         """
-        log.debug('open_write(%s): start', key)
+        log.debug('started with %s', key)
 
         if key.endswith(TEMP_SUFFIX):
             raise ValueError('Keys must not end with %s' % TEMP_SUFFIX)
 
+        headers = CaseInsensitiveDict()
         if metadata is None:
             metadata = dict()
-        elif not isinstance(metadata, dict):
-            raise TypeError('*metadata*: expected dict or None, got %s' % type(metadata))
-
-        headers = CaseInsensitiveDict()
         self._add_meta_headers(headers, metadata)
 
         return ObjectW(key, self, headers)
 
-    @copy_ancestor_docstring
-    def clear(self):
-
-        # We have to cache keys, because otherwise we can't use the
-        # http connection to delete keys.
-        for (no, s3key) in enumerate(list(self)):
-            if no != 0 and no % 1000 == 0:
-                log.info('clear(): deleted %d objects so far..', no)
-
-            log.debug('clear(): deleting key %s', s3key)
-
-            # Ignore missing objects when clearing backend
-            self.delete(s3key, True)
-
     @retry
     @copy_ancestor_docstring
     def delete(self, key, force=False):
         if key.endswith(TEMP_SUFFIX):
             raise ValueError('Keys must not end with %s' % TEMP_SUFFIX)
-        log.debug('delete(%s)', key)
+        log.debug('started with %s', key)
         try:
             resp = self._do_request('DELETE', '/%s%s' % (self.prefix, key))
             self._assert_empty_response(resp)
@@ -434,14 +392,18 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             elif exc.status != 404:
                 raise
 
+    # Temporary hack. I promise to make this nicer by the next
+    # release :-).
     @retry
+    def _copy_helper(self, method, path, headers):
+        self._do_request(method, path, headers=headers)
+        self.conn.discard()
+
     @copy_ancestor_docstring
     def copy(self, src, dest, metadata=None):
-        log.debug('copy(%s, %s): start', src, dest)
+        log.debug('started with %s, %s', src, dest)
         if dest.endswith(TEMP_SUFFIX) or src.endswith(TEMP_SUFFIX):
             raise ValueError('Keys must not end with %s' % TEMP_SUFFIX)
-        if not (metadata is None or isinstance(metadata, dict)):
-            raise TypeError('*metadata*: expected dict or None, got %s' % type(metadata))
 
         headers = CaseInsensitiveDict()
         headers['X-Copy-From'] = '/%s/%s%s' % (self.container_name, self.prefix, src)
@@ -456,8 +418,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             headers['X-Delete-After'] = '600'
 
         try:
-            self._do_request('PUT', '/%s%s' % (self.prefix, dest), headers=headers)
-            self.conn.discard()
+            self._copy_helper('PUT', '/%s%s' % (self.prefix, dest), headers)
         except HTTPError as exc:
             if exc.status == 404:
                 raise NoSuchObject(src)
@@ -469,21 +430,17 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
         # Update metadata
         headers = CaseInsensitiveDict()
         self._add_meta_headers(headers, metadata)
-        self._do_request('POST', '/%s%s' % (self.prefix, dest), headers=headers)
-        self.conn.discard()
+        self._copy_helper('POST', '/%s%s' % (self.prefix, dest), headers)
 
         # Rename object
         headers = CaseInsensitiveDict()
         headers['X-Copy-From'] = '/%s/%s%s' % (self.container_name, self.prefix, dest)
-        self._do_request('PUT', '/%s%s' % (self.prefix, final_dest), headers=headers)
-        self.conn.discard()
+        self._copy_helper('PUT', '/%s%s' % (self.prefix, final_dest), headers)
 
+    @retry
     @copy_ancestor_docstring
     def update_meta(self, key, metadata):
-        log.debug('start for %s', key)
-        if not isinstance(metadata, dict):
-            raise TypeError('*metadata*: expected dict, got %s' % type(metadata))
-
+        log.debug('started with %s', key)
         headers = CaseInsensitiveDict()
         self._add_meta_headers(headers, metadata)
         self._do_request('POST', '/%s%s' % (self.prefix, key), headers=headers)
@@ -492,14 +449,14 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     @retry_generator
     @copy_ancestor_docstring
     def list(self, prefix='', start_after='', batch_size=5000):
-        log.debug('list(%s, %s): start', prefix, start_after)
+        log.debug('started with %s, %s', prefix, start_after)
 
         keys_remaining = True
-        marker = start_after
+        marker = self.prefix + start_after
         prefix = self.prefix + prefix
 
         while keys_remaining:
-            log.debug('list(%s): requesting with marker=%s', prefix, marker)
+            log.debug('requesting with marker=%s', marker)
 
             try:
                 resp = self._do_request('GET', '/', query_string={'prefix': prefix,
@@ -543,12 +500,6 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     def close(self):
         self.conn.disconnect()
 
-    @copy_ancestor_docstring
-    def reset(self):
-        if self.conn.response_pending() or self.conn._out_remaining:
-            log.debug('Resetting state of http connection %d', id(self.conn))
-            self.conn.disconnect()
-
 class AuthenticationExpired(Exception):
     '''Raised if the provided Authentication Token has expired'''
 
