@@ -9,6 +9,8 @@ This program can be distributed under the terms of the GNU GPLv3.
 from .logging import logging, setup_logging, QuietError
 from . import CURRENT_FS_REV, BUFSIZE, CTRL_INODE, PICKLE_PROTOCOL, ROOT_INODE
 from .backends.common import NoSuchObject
+from .backends.comprenc import ComprencBackend
+from .backends.local import Backend as LocalBackend
 from .common import (inode_for_path, sha256_fh, get_path, get_backend_cachedir,
                      get_seq_no, stream_write_bz2, stream_read_bz2, is_mounted,
                      get_backend, pretty_print_size)
@@ -19,8 +21,10 @@ from os.path import basename
 import apsw
 import os
 import pickle
+import hashlib
 import re
 import shutil
+import itertools
 import stat
 import sys
 import tempfile
@@ -81,12 +85,14 @@ class Fsck(object):
             self.check_contents_inode()
             self.check_contents_parent_inode()
 
+            self.check_objects_temp()
             self.check_objects_refcount()
             self.check_objects_id()
             self.check_objects_size()
 
             self.check_blocks_obj_id()
             self.check_blocks_refcount()
+            self.check_blocks_checksum()
 
             self.check_inode_blocks_block_id()
             self.check_inode_blocks_inode()
@@ -110,7 +116,9 @@ class Fsck(object):
     def log_error(self, *a, **kw):
         '''Log file system error if not expected'''
 
-        if not self.expect_errors:
+        if self.expect_errors:
+            return log.debug(*a, **kw)
+        else:
             return log.warning(*a, **kw)
 
     def check_foreign_keys(self):
@@ -632,6 +640,32 @@ class Fsck(object):
             self.conn.execute('DROP TABLE refcounts')
             self.conn.execute('DROP TABLE IF EXISTS wrong_refcounts')
 
+    def check_blocks_checksum(self):
+        """Check blocks.hash"""
+
+        log.info('Checking blocks (checksums)...')
+
+        for (block_id, obj_id) in self.conn.query('SELECT id, obj_id FROM blocks '
+                                                  'WHERE hash IS NULL'):
+            self.found_errors = True
+            self.log_error("No cached checksum for block %d, recomputing...", block_id)
+
+            # At the moment, we support only one block per object
+            assert self.conn.get_val('SELECT refcount FROM objects WHERE id=?',
+                                     (obj_id,)) == 1
+
+            def do_read(fh):
+                sha = hashlib.sha256()
+                while True:
+                    buf = fh.read(BUFSIZE)
+                    if not buf:
+                        break
+                    sha.update(buf)
+                return sha.digest()
+            hash_ = self.backend.perform_read(do_read, "s3ql_data_%d" % obj_id)
+
+            self.conn.execute('UPDATE blocks SET hash=? WHERE id=?',
+                              (hash_, block_id))
 
     def create_inode(self, mode, uid=os.getuid(), gid=os.getgid(),
                      mtime=None, atime=None, ctime=None, refcount=None,
@@ -821,6 +855,32 @@ class Fsck(object):
         self.conn.execute("DELETE FROM objects WHERE refcount = 0")
 
 
+    def check_objects_temp(self):
+        """Remove temporary objects"""
+
+        # Tests may provide a plain backend directly, but in regular operation
+        # we'll always work with a ComprencBackend (even if there is neiter
+        # compression nor encryption)
+        if isinstance(self.backend, ComprencBackend):
+            plain_backend = self.backend.backend
+        else:
+            assert isinstance(self.backend, LocalBackend)
+            plain_backend = self.backend
+
+        if not isinstance(plain_backend, LocalBackend):
+            return
+
+        log.info('Checking for temporary objects (backend)...')
+
+        for (path, dirnames, filenames) in os.walk(plain_backend.prefix, topdown=True):
+            for name in filenames:
+                if not re.search(r'^[^#]+#[0-9]+--?[0-9]+\.tmp$', name):
+                    continue
+
+                self.found_errors = True
+                self.log_error("removing temporary file %s", name)
+                os.unlink(os.path.join(path, name))
+
     def check_objects_id(self):
         """Check objects.id"""
 
@@ -830,13 +890,18 @@ class Fsck(object):
                                    "WHERE name=? AND parent_inode=?", (b"lost+found", ROOT_INODE))
 
         # We use this table to keep track of the objects that we have seen
+        if sys.stdout.isatty():
+            stamp1 = 0
+        else:
+            stamp1 = float('inf')
         self.conn.execute("CREATE TEMP TABLE obj_ids (id INTEGER PRIMARY KEY)")
         try:
             for (i, obj_name) in enumerate(self.backend.list('s3ql_data_')):
-
-                if i % 500 == 0 and sys.stdout.isatty():
+                stamp2 = time.time()
+                if stamp2 - stamp1 > 1:
                     sys.stdout.write('\r..processed %d objects so far..' % i)
                     sys.stdout.flush()
+                    stamp1 = stamp2
 
                 # We only bother with data objects
                 try:
@@ -1010,6 +1075,7 @@ class ROFsck(Fsck):
 
             self.check_blocks_obj_id()
             self.check_blocks_refcount()
+            self.check_blocks_checksum()
 
             self.check_inode_blocks_block_id()
             self.check_inode_blocks_inode()
@@ -1030,6 +1096,15 @@ class ROFsck(Fsck):
             log.info('Dropping temporary indices...')
             self.conn.execute('ROLLBACK')
 
+    def check_blocks_checksum(self):
+        """Check blocks.hash"""
+
+        log.info('Checking blocks (checksums)...')
+
+        for (block_id,) in self.conn.query('SELECT id FROM blocks WHERE hash IS NULL'):
+            self.found_errors = True
+            self.log_error("No cached checksum for block %d!", block_id)
+
     def check_objects_size(self):
         """Check objects.size"""
 
@@ -1037,10 +1112,7 @@ class ROFsck(Fsck):
 
         for (obj_id,) in self.conn.query('SELECT id FROM objects WHERE size IS NULL'):
             self.found_errors = True
-            self.log_error("Object %d has no size information, setting to zero...", obj_id)
-
-            self.conn.execute('UPDATE objects SET size=? WHERE id=?', (0, obj_id))
-
+            self.log_error("Object %d has no size information!", obj_id)
 
 def parse_args(args):
 
@@ -1140,6 +1212,17 @@ def main(args=None):
         param['seq_no'] = seq_no
         param['needs_fsck'] = True
 
+    if not db and os.path.exists(cachepath + '-cache'):
+        for i in itertools.count():
+            bak_name = '%s-cache.bak%d' % (cachepath, i)
+            if not os.path.exists(bak_name):
+                break
+        log.warning('Found outdated cache directory (%s), renaming to .bak%d',
+                    cachepath + '-cache', i)
+        log.warning('You should delete this directory once you are sure that '
+                    'everything is in order.')
+        os.rename(cachepath + '-cache', bak_name)
+
     if (not param['needs_fsck']
         and param['max_inode'] < 2 ** 31
         and (time.time() - param['last_fsck'])
