@@ -12,9 +12,6 @@ from .backends.pool import BackendPool
 from .block_cache import BlockCache
 from .common import (get_backend_cachedir, get_seq_no, stream_write_bz2, stream_read_bz2,
                      get_backend_factory, pretty_print_size)
-from .backends.comprenc import ComprencBackend
-from .backends.common import CorruptedObjectError
-from .adm import upgrade_monkeypatch
 from .daemonize import daemonize
 from .database import Connection
 from .inode_cache import InodeCache
@@ -26,6 +23,7 @@ import _thread
 import argparse
 import faulthandler
 import llfuse
+import itertools
 import os
 import platform
 import subprocess
@@ -390,22 +388,7 @@ def get_metadata(backend, cachepath):
             log.info('Using cached metadata.')
             db = Connection(cachepath + '.db')
     else:
-        try:
-            param = backend.lookup('s3ql_metadata')
-        except CorruptedObjectError:
-            if not isinstance(backend, ComprencBackend):
-                raise
-            # Temporary band-aid to identify old file system revision
-            upgrade_monkeypatch(backend)
-            try:
-                param = backend.lookup('s3ql_metadata')
-            except:
-                pass
-            else:
-                if param['revision'] >= CURRENT_FS_REV:
-                    raise
-            finally:
-                del backend._verify_meta
+        param = backend.lookup('s3ql_metadata')
 
     # Check for unclean shutdown
     if param['seq_no'] < seq_no:
@@ -434,6 +417,17 @@ def get_metadata(backend, cachepath):
     elif param['max_inode'] > 2 ** 31:
         log.warning('Few free inodes remaining, running fsck is recommended')
 
+    if os.path.exists(cachepath + '-cache'):
+        for i in itertools.count():
+            bak_name = '%s-cache.bak%d' % (cachepath, i)
+            if not os.path.exists(bak_name):
+                break
+        log.warning('Found outdated cache directory (%s), renaming to .bak%d',
+                    cachepath + '-cache', i)
+        log.warning('You should delete this directory once you are sure that '
+                    'everything is in order.')
+        os.rename(cachepath + '-cache', bak_name)
+
     # Download metadata
     if not db:
         with tempfile.TemporaryFile() as tmpfh:
@@ -647,7 +641,7 @@ class MetadataUploadThread(Thread):
         self.fs = None
 
     def run(self):
-        log.debug('MetadataUploadThread: start')
+        log.debug('started')
 
         assert self.fs is not None
 
@@ -704,12 +698,12 @@ class MetadataUploadThread(Thread):
         # Break reference loop
         self.fs = None
 
-        log.debug('MetadataUploadThread: end')
+        log.debug('finished')
 
     def stop(self):
         '''Signal thread to terminate'''
 
-        log.debug('MetadataUploadThread: stop() called')
+        log.debug('started')
         self.quit = True
         self.event.set()
 
@@ -765,7 +759,7 @@ class CommitThread(Thread):
         self.name = 'CommitThread'
 
     def run(self):
-        log.debug('CommitThread: start')
+        log.debug('started')
 
         while not self.stop_event.is_set():
             did_sth = False
@@ -798,14 +792,13 @@ class CommitThread(Thread):
             if not did_sth:
                 self.stop_event.wait(5)
 
-        log.debug('CommitThread: end')
+        log.debug('finished')
 
     def stop(self):
         '''Signal thread to terminate'''
 
-        log.debug('CommitThread: stop() called')
+        log.debug('started')
         self.stop_event.set()
 
-
 if __name__ == '__main__':
     main(sys.argv[1:])
