@@ -14,13 +14,14 @@ if __name__ == '__main__':
 
 from contextlib import contextmanager
 from s3ql.backends import local
-from s3ql.backends.common import AbstractBackend, NoSuchObject
+from s3ql.backends.common import AbstractBackend
 from s3ql.backends.pool import BackendPool
-from s3ql.block_cache import BlockCache, QuitSentinel, NoWorkerThreads
+from s3ql.block_cache import BlockCache, QuitSentinel
 from s3ql.mkfs import init_tables
 from s3ql.metadata import create_tables
 from s3ql.database import Connection
-from common import AsyncFn, catch_logmsg, safe_sleep
+from s3ql.common import AsyncFn
+from common import catch_logmsg, safe_sleep
 import llfuse
 import errno
 import os
